@@ -15,10 +15,11 @@ import pkgutil
 
 tf.disable_v2_behavior()
 
+
 # from other .py scripts
 if pkgutil.find_loader("UNet") is not None:
-    from toolbox.imtools import *
-    from toolbox.ftools import *
+    from toolbox.imtools import tifread, normalize, im2double, imwrite
+    from toolbox.ftools import pathjoin, saveData, loadData
     from toolbox.PartitionOfImage import PI2D
     from toolbox.GPUselect import pick_gpu_lowest_memory
 else:
@@ -46,6 +47,7 @@ else:
 #from .toolbox.ftools import fileparts, listfiles, listsubdirs, pathjoin, saveData, loadData, createFolderIfNonExistent, moveFile, copyFile, removeFile
 
 
+
 # Functions 
 def concat3 (lst):
     return tf.concat(lst, 3)
@@ -321,7 +323,6 @@ class UNet2D:
 		WValid = np.zeros((nValid, imSize, imSize, nClasses))
 		WTest = np.zeros((nTest, imSize, imSize, nClasses))
 
-		print('loading data, computing mean / st dev')
 		if not os.path.exists(modelPath):
 			os.makedirs(modelPath)
 		# if restoreVariables:
@@ -793,4 +794,5 @@ class UNet2D:
 					PI2D.patchOutput(i - j + k, pm)
 		# PI2D.patchOutput(i-j+k,normalize(imgradmag(PI2D.getPatch(i-j+k),1)))
 
-		return PI2D.getValidOutput()
\ No newline at end of file
+		return PI2D.getValidOutput()
+
