@@ -8,4 +8,7 @@ from .gator import gator
 from .gatorPhenotype import gatorPhenotype
 from .gatorTrain import gatorTrain
 from .gatorPredict import gatorPredict
-from .addPredictions import addPredictions
\ No newline at end of file
+from .addPredictions import addPredictions
+from .UNet import concat3, UNet2D
+from .gatorPipeline import gatorPipeline
+from .gatorExport import gatorExport
\ No newline at end of file
