@@ -35,7 +35,7 @@ import argparse
 
 def gator (gatorObject,
            gatorScore='gatorScore',
-           minAbundance=0.002,
+           minAbundance=0.005,
            percentiles=[1, 20, 80, 99],
            dropMarkers = None,
            RobustScale=False,
@@ -47,8 +47,8 @@ def gator (gatorObject,
            random_state=0,
            rescaleMethod='minmax',
            label='gatorOutput',
-           silent=True,
-           outputDir=None, **kwargs):
+           verbose=True,
+           projectDir=None, **kwargs):
     """
 Parameters:
     gatorObject (anndata):
@@ -81,7 +81,7 @@ Parameters:
         Apply `log1p` transformation on the data, unless it has already been log
         transformed in which case set it to `False`. 
 
-    stringentThreshold (float, optional):
+    stringentThreshold (bool, optional):
         The Gaussian Mixture Model (GMM) is utilized to distinguish positive and 
         negative cells by utilizing gatorScores. The stringentThreshold can be utilized 
         to further refine the classification of positive and negative cells. 
@@ -110,20 +110,20 @@ Parameters:
         Assign a label for the object within `adata.uns` where the predictions
         from Gator will be stored. 
 
-    silent (bool, optional):
-        Enables the display of step-by-step events on the console. 
+    verbose (bool, optional):
+        If True, print detailed information about the process to the console.  
 
-    outputDir (str, optional):
+    projectDir (str, optional):
         Provide the path to the output directory. The result will be located at
-        `outputDir/GATOR/gatorOutput/`. 
+        `projectDir/GATOR/gatorOutput/`. 
 
     **kwargs (keyword parameters):
         Additional arguments to pass to the `HistGradientBoostingClassifier()` function.
 
 Returns:
     gatorObject (anndata):
-        If outputDir is provided the updated Gator Object will saved within the
-        provided outputDir.
+        If projectDir is provided the updated Gator Object will saved within the
+        provided projectDir.
 
 Example:
 
@@ -147,11 +147,11 @@ Example:
                     random_state=0,
                     rescaleMethod='sigmoid',
                     label='gatorOutput',
-                    silent=True,
-                    outputDir=cwd)
+                    verbose=True,
+                    projectDir=cwd)
         
         # Same function if the user wants to run it via Command Line Interface
-        python gator.py --gatorObject /Users/aj/Desktop/gatorExampleData/GATOR/gatorObject/exampleImage_gatorPredict.ome.h5ad --outputDir /Users/aj/Desktop/gatorExampleData
+        python gator.py --gatorObject /Users/aj/Desktop/gatorExampleData/GATOR/gatorObject/exampleImage_gatorPredict.ome.h5ad --projectDir /Users/aj/Desktop/gatorExampleData
                 
         
         ```
@@ -159,7 +159,7 @@ Example:
     """
 
     #gatorObject = "/Users/aj/Dropbox (Partners HealthCare)/Data/gator/data/ajn_training_data/GATOR/gatorObject/1_6_GatorOutput.h5ad"
-    #gatorScore='gatorScore'; minAbundance=0.002; percentiles=[1, 20, 80, 99]; dropMarkers = None;leakData=False;rescaleMethod='sigmoid';silent = False
+    #gatorScore='gatorScore'; minAbundance=0.002; percentiles=[1, 20, 80, 99]; dropMarkers = None;leakData=False;rescaleMethod='sigmoid';verbose = False
     #scaleData=False; log=True; x_coordinate='X_centroid'; y_coordinate='Y_centroid'; imageid='imageid'; random_state=0; label='gatorOutput'
     #gatorObject = '/Users/aj/Dropbox (Partners HealthCare)/Data/gator/data/ajn_training_data/GATOR/gatorPredict/2_28_GatorOutput.h5ad'
     #gatorObject = '/Users/aj/Dropbox (Partners HealthCare)/Data/gator/data/ajn_training_data/GATOR/gatorPredict/4_113_GatorOutput.h5ad'
@@ -197,6 +197,39 @@ Example:
             if num_rows_with_high_values / len(df) < minAbundance:
                 columns_to_keep.append(column)
         return columns_to_keep
+    
+    # count the number of pos and neg elements in a list
+    def count_pos_neg(lst):
+        arr = np.array(lst)
+        result = {'pos': np.sum(arr == 'pos'), 'neg': np.sum(arr == 'neg')}
+        result['pos'] = result['pos'] if result['pos'] > 0 else 0
+        result['neg'] = result['neg'] if result['neg'] > 0 else 0
+        return result
+    
+    # alternative to find if markers failed
+    def simpleGMM_failedMarkers (df, n_components, minAbundance, random_state):
+        # prepare data
+        columns_to_keep = []
+        for column in df.columns:
+            #print(str(column))
+            colValue = df[[column]].values  
+            colValue[0] = 0; colValue[1] = 1;   # force the model to converge from 0-1
+            gmm = GaussianMixture(n_components=n_components,  random_state=random_state)
+            gmm.fit(colValue)
+            predictions = gmm.predict(colValue)
+            # Get the mean of each Gaussian component
+            means = gmm.means_.flatten()
+            # Sort the mean values in ascending order
+            sorted_means = np.sort(means)
+            # Assign 'pos' to rows with higher mean distribution and 'neg' to rows with lower mean distribution
+            labels = np.where(predictions == np.argmax(means), 'pos', 'neg')
+            # count pos and neg
+            counts = count_pos_neg(labels)
+            # find if the postive cells is less than the user defined min abundance
+            if counts['pos'] / len(df)  < minAbundance:
+                columns_to_keep.append(column)
+        return columns_to_keep
+
 
     # preprocess data (step-4)
     def pre_process (data, log=log):
@@ -384,12 +417,19 @@ Example:
     ###########################################################################
     # step-1 : Identify markers that have failed in this dataset
     ###########################################################################
-
-    failed_markers = get_columns_with_low_values (df=adata.uns[gatorScore],minAbundance=minAbundance)
+    # 0ld thresholding method
+    #failed_markers = get_columns_with_low_values (df=adata.uns[gatorScore],minAbundance=minAbundance)
+    
+    # New GMM method
+    failed_markers = simpleGMM_failedMarkers (df=adata.uns[gatorScore], 
+                                              n_components=2, 
+                                              minAbundance=minAbundance, 
+                                              random_state=random_state)
+    
     # to store in adata
     failed_markers_dict = {adata.obs[imageid].unique()[0] : failed_markers}
 
-    if silent is False:
+    if verbose is True:
         print('Failed Markers are: ' + ", ".join(str(x) for x in failed_markers))
 
     ###########################################################################
@@ -434,7 +474,7 @@ Example:
 
 
         if marker in expression_unet_common:
-            if silent is False:
+            if verbose is True:
                 print("NN marker: " + str(marker))
             # run GMM on probQuant_data
             X = probQuant_data[marker].values.reshape(-1,1)
@@ -466,18 +506,18 @@ Example:
             neg = list(set(expCellsNeg).intersection(set(probCellsNeg)))
 
             # print no of cells
-            if silent is False:
+            if verbose is True:
                 print("POS cells: {} and NEG cells: {}.".format(len(pos), len(neg)))
 
             # check if the length is less than 20 cells and if so add the marker to only_expression
             if len(pos) < 20 or len(neg) < 20: ## CHECK!
                 only_expression.append(marker)
-                if silent is False:
+                if verbose is True:
                     print ("As the number of POS/NEG cells is low for " + str(marker) + ", GMM will fitted using only expression values.")
 
 
         if marker in only_expression:
-            if silent is False:
+            if verbose is True:
                 print("Expression marker: " + str(marker))
             # Run GMM only on the expression data
             Z = pre_processed_data[marker].values.reshape(-1,1)
@@ -503,7 +543,7 @@ Example:
             random.seed(random_state); neg = random.sample(neg, k=int(len(neg) * 0.7))
 
             # print no of cells
-            if silent is False:
+            if verbose is True:
                 print("Defined POS cells is {} and NEG cells is {}.".format(len(pos), len(neg)))
 
             # What happens of POS/NEG is less than 20
@@ -513,14 +553,14 @@ Example:
                     percentiles = [1,20,80,99]
                 neg = list(indexPercentile (pre_processed_data, marker, lowPercentile=percentiles[0], highPercentile=percentiles[1]))
                 pos = list(indexPercentile (pre_processed_data, marker, lowPercentile=percentiles[2], highPercentile=percentiles[3]))
-                if silent is False:
+                if verbose is True:
                     print ("As the number of POS/NEG cells is low for " + str(marker) + ", cells falling within the given percentile " + str(percentiles) + ' was used.')
 
         # return the output
         return marker, pos, neg
 
     # Run the function on all markers
-    if silent is False:
+    if verbose is True:
         print("Intial GMM Fitting")
     r_bonafide_cells = lambda x: bonafide_cells (marker=x,
                                                 expression_unet_common=expression_unet_common,
@@ -545,7 +585,7 @@ Example:
         neg = bonafide_cells_result[2]
         PD = pre_processed_data.copy()
 
-        if silent is False:
+        if verbose is True:
             print('Processing: ' + str(marker))
 
         # class balance the number of pos and neg cells based on the lowest denominator
@@ -572,7 +612,7 @@ Example:
         return marker, combined_data
 
     # Run the function
-    if silent is False:
+    if verbose is True:
         print("Building the Training Data")
     r_trainingData = lambda x: trainingData (bonafide_cells_result=x,
                                                  pre_processed_data=pre_processed_data,
@@ -596,7 +636,7 @@ Example:
         predictionData = combined_data.drop(index=index_names_to_drop, inplace=False)
         predictionData = predictionData.drop('label', axis=1)
 
-        if silent is False:
+        if verbose is True:
             print('classifying: ' + str(marker))
 
         # shuffle the data
@@ -634,7 +674,7 @@ Example:
         return marker, pred, prob, midpoint
 
     # Run the function
-    if silent is False:
+    if verbose is True:
         print("Fitting model for classification:")
     r_gatorClassifier = lambda x: gatorClassifier (trainingData_result=x,random_state=random_state)
     gatorClassifier_result = list(map(r_gatorClassifier, trainingData_result))
@@ -675,7 +715,7 @@ Example:
         pos = bonafide_cells_result[1]
         neg = bonafide_cells_result[2]
 
-        if silent is False:
+        if verbose is True:
             print("Processing: " + str(marker))
         # prepare data
         X = pre_processed_data.drop(marker, axis=1)
@@ -702,7 +742,7 @@ Example:
         return results
 
     # Run the function
-    if silent is False:
+    if verbose is True:
         print("Running Anomaly Detection")
     r_anomalyDetector = lambda x: anomalyDetector (bonafide_cells_result = x,
                                                    pre_processed_data = pre_processed_data,
@@ -744,7 +784,7 @@ Example:
 
     # marker = 'ECAD'
     def rescaleData (marker, pre_processed_data, prediction_results, midpoints_dict):
-        if silent is False:
+        if verbose is True:
             print("Processing: " + str(marker))
         # unravel data
         data = pre_processed_data[marker].values
@@ -767,7 +807,7 @@ Example:
         return rescaled_data
 
     # Run the function
-    if silent is False:
+    if verbose is True:
         print("Rescaling the raw data")
     r_rescaleData = lambda x: rescaleData (marker=x,
                                            pre_processed_data=pre_processed_data,
@@ -806,14 +846,15 @@ Example:
     # add the pre-processed data as a layer
     bdata.layers["preProcessed"] = pre_processed_data
     bdata.uns = adata.uns
+    bdata.uns['failedMarkers'] = failed_markers_dict
 
     # save the prediction results in anndata object
     bdata.uns[str(label)] = prediction_results
     #bdata.uns[str(label)] = probability_results
 
     # Save data if requested
-    if outputDir is not None:
-        finalPath = pathlib.Path(outputDir + '/GATOR/gatorOutput')
+    if projectDir is not None:
+        finalPath = pathlib.Path(projectDir + '/GATOR/gatorOutput')
         if not os.path.exists(finalPath):
             os.makedirs(finalPath)
         if len(gatorObjectPath) > 1:
@@ -821,7 +862,11 @@ Example:
         else:
             imid = gatorObjectPath[0].stem
         bdata.write(finalPath / f'{imid}.h5ad')
-
+        
+    # Finish Job
+    if verbose is True:
+        print('Gator ran successfully, head over to "' + str(projectDir) + '/GATOR/gatorOutput" to view results')
+        
     return bdata
 
 # Make the Function CLI compatable
@@ -834,14 +879,15 @@ if __name__ == '__main__':
     parser.add_argument('--dropMarkers', type=list, default=None, help='Specify a list of markers to be removed from the analysis')
     parser.add_argument('--RobustScale', type=bool, default=False, help='When set to True, the data will be subject to Robust Scaling before the Gradient Boosting Classifier is trained')
     parser.add_argument('--log', type=bool, default=True, help='Apply `log1p` transformation on the data, unless it has already been log transformed in which case set it to `False`')
+    parser.add_argument('--stringentThreshold', type=bool, default=False, help='Threshold to refine the classification of positive and negative cells')
     parser.add_argument('--x_coordinate', type=str, default='X_centroid', help='The column name in `single-cell spatial table` that records the X coordinates for each cell')
     parser.add_argument('--y_coordinate', type=str, default='Y_centroid', help='The column name in `single-cell spatial table` that records the Y coordinates for each cell')
     parser.add_argument('--imageid', type=str, default='imageid', help='The name of the column that holds the unique image ID')
     parser.add_argument('--random_state', type=int, default=0, help='Seed used by the random number generator')
     parser.add_argument('--rescaleMethod', type=str, default='minmax', help='Choose between `sigmoid` and `minmax`')
     parser.add_argument('--label', type=str, default='gatorOutput', help='Assign a label for the object within `adata.uns` where the predictions from Gator will be stored')
-    parser.add_argument('--silent', type=bool, default=True, help='Enables the display of step-by-step events on the console')
-    parser.add_argument('--outputDir', type=str, default=None, help='Provide the path to the output directory')
+    parser.add_argument('--verbose', type=bool, default=True, help='Enables the display of step-by-step events on the console')
+    parser.add_argument('--projectDir', type=str, default=None, help='Provide the path to the output directory')
     args = parser.parse_args()
     gator(gatorObject=args.gatorObject,
           gatorScore=args.gatorScore,
@@ -850,11 +896,12 @@ if __name__ == '__main__':
           dropMarkers=args.dropMarkers,
           RobustScale=args.RobustScale,
           log=args.log,
+          stringentThreshold=args.stringentThreshold, 
           x_coordinate=args.x_coordinate,
           y_coordinate=args.y_coordinate,
           imageid=args.imageid,
           random_state=args.random_state,
           rescaleMethod=args.rescaleMethod,
           label=args.label,
-          silent=args.silent,
-          outputDir=args.outputDir)
+          verbose=args.verbose,
+          projectDir=args.projectDir)
